// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include "config.h"
#ifdef WITH_GWAVI
extern "C" {
#include <gwavi.h>
}
#include <unistd.h>

#include "target_avi.h"
#include "error.h"
#include "exec.h"
#include "log.h"
#include "picio.h"
#include "utils.h"

target_avi::target_avi(const std::string & id, const std::string & descr, source *const s, const std::string & store_path, const std::string & prefix, const std::string & fmt, const int quality, const int max_time, const double interval, const std::vector<filter *> *const filters, const std::string & exec_start, const std::string & exec_cycle, const std::string & exec_end, const double override_fps, configuration_t *const cfg, const bool is_view_proxy, const bool use_previous_when_fail) : target(id, descr, s, store_path, prefix, fmt, max_time, interval, filters, exec_start, exec_cycle, exec_end, override_fps, cfg, is_view_proxy, use_previous_when_fail), quality(quality)
{
	if (this -> descr == "")
		this -> descr = store_path + "/" + prefix;
}

target_avi::~target_avi()
{
	stop();
}

void target_avi::operator()()
{
	set_thread_name("storea_" + prefix);

	s -> register_user();

	time_t cut_ts = time(nullptr) + max_time;

	uint64_t prev_ts = 0;
	bool is_start = true;
	std::string name;
	unsigned f_nr = 0;

	uint8_t *prev_frame = nullptr;

	struct gwavi_t *gwavi = nullptr;

	for(;!local_stop_flag;) {
		pauseCheck();

		double before_ts = get_ts();

		int w = -1, h = -1;
		uint8_t *work = nullptr;
		size_t work_len = 0;
		if (s -> get_frame((!filters || filters -> empty()) ? E_JPEG : E_RGB, quality, &prev_ts, &w, &h, &work, &work_len, use_previous_when_fail)) {
			if (max_time > 0 && time(nullptr) >= cut_ts) {
				log(id, LL_DEBUG, "new file");

				if (gwavi)
					gwavi_close(gwavi);
				gwavi = nullptr;

				cut_ts = time(nullptr) + max_time;
			}

			if (!gwavi) {
				name = gen_filename(s, fmt, store_path, prefix, "avi", get_us(), f_nr++);
				create_path(name);

				if (!exec_start.empty() && is_start) {
					exec(exec_start, name);
					is_start = false;
				}
				else if (!exec_cycle.empty()) {
					exec(exec_cycle, name);
				}

				int fps = interval <= 0 ? 25 : std::max(1, int(1.0 / interval));

				register_file(name);

				gwavi = gwavi_open((char *)name.c_str(), w, h, (char *)"MJPG", override_fps > 0 ? override_fps : fps, nullptr);
				if (!gwavi)
					error_exit(true, "Cannot create %s", name.c_str());

				if (pre_record) {
					bool ok = true;

					log(id, LL_DEBUG_VERBOSE, "Write pre-recorded frames");

					for(frame_t pair : *pre_record) {
						if (pair.e == E_JPEG)
							gwavi_add_frame(gwavi, pair.data, pair.len);
						else {
							char *data = nullptr;
							size_t data_size = 0;
							if (!write_JPEG_memory(s -> get_meta(), w, h, quality, pair.data, &data, &data_size)) {
								ok = false;
								break;
							}

							gwavi_add_frame(gwavi, (unsigned char *)data, data_size);

							free(data);
						}

						free(pair.data);
					}

					delete pre_record;
					pre_record = nullptr;

					if (!ok)
						break;
				}
			}

			log(id, LL_DEBUG_VERBOSE, "Write frame");
			if (!filters || filters -> empty())
				gwavi_add_frame(gwavi, work, work_len);
			else {
				source *cur_s = is_view_proxy ? ((view *)s) -> get_current_source() : s;
				instance_t *inst = find_instance_by_interface(cfg, cur_s);

				apply_filters(inst, cur_s, filters, prev_frame, work, prev_ts, w, h);

				char *data = nullptr;
				size_t data_size = 0;
				bool ok = write_JPEG_memory(s -> get_meta(), w, h, quality, work, &data, &data_size);
				gwavi_add_frame(gwavi, (unsigned char *)data, data_size);
				free(data);

				if (!ok)
					break;
			}

			free(prev_frame);
			prev_frame = work;
		}

		handle_fps(before_ts);
	}

	if (gwavi)
		gwavi_close(gwavi);

	free(prev_frame);

	if (!exec_end.empty())
		exec(exec_end, name);

	s -> unregister_user();

	log(id, LL_INFO, "stopping");
}
#endif
